﻿// -----------------------------------------------------------------------
//  <copyright file="RESTService.svc.cs" company="Microsoft">
//      Copyright (c) Microsoft. All rights reserved.
//  </copyright>
// -----------------------------------------------------------------------

namespace SimpleAuthService
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Security.Claims;
    using System.Text;
    using System.Threading;

    using Microsoft.XboxLive.Auth;
    using Microsoft.XboxLive.Auth.Claims;

    public class RESTService : IRESTService
    {
        public ClaimsOutput GetClaims()
        {
            ClaimsOutput claimOutput = new ClaimsOutput
            {
                UserClaims = new List<Identity>(),
                DeviceClaim = new List<Identity>(),
                TitleClaim = new List<Identity>(),
                AggregateClaims = new List<Identity>()
            };

            //Grab claims from the principal that was present in the "Authorization" Header in the request
            ClaimsPrincipal principal = Thread.CurrentPrincipal as ClaimsPrincipal;

            if (principal != null)
            {
                IEnumerable<ClaimsIdentity> userIdentities = principal.GetUserIdentities();
                ClaimsIdentity titleIdentity = principal.GetTitleIdentity();
                ClaimsIdentity deviceIdentity = principal.GetDeviceIdentity();
                ClaimsIdentity aggregateIdentity = principal.GetAggregateIdentity();

                //Step through each user identity claim.
                foreach (ClaimsIdentity userIdentity in userIdentities)
                {
                    Identity identity = new Identity { ClaimsList = new List<StringClaim>() };
                    foreach (Claim c in userIdentity.Claims)
                    {
                        identity.ClaimsList.Add(new StringClaim { ClaimType = c.Type, Value = c.Value });
                    }

                    claimOutput.UserClaims.Add(new Identity { ClaimsList = identity.ClaimsList });
                }

                if (deviceIdentity != null)
                {
                    Identity identity = new Identity { ClaimsList = new List<StringClaim>() };
                    foreach (Claim c in deviceIdentity.Claims)
                    {
                        identity.ClaimsList.Add(new StringClaim { ClaimType = c.Type, Value = c.Value });
                    }
                    claimOutput.DeviceClaim.Add(new Identity { ClaimsList = identity.ClaimsList });
                }

                if (titleIdentity != null)
                {
                    Identity identity = new Identity { ClaimsList = new List<StringClaim>() };
                    foreach (Claim c in titleIdentity.Claims)
                    {
                        identity.ClaimsList.Add(new StringClaim { ClaimType = c.Type, Value = c.Value });
                    }

                    claimOutput.TitleClaim.Add(new Identity { ClaimsList = identity.ClaimsList });
                }

                if (aggregateIdentity != null)
                {
                    Identity identity = new Identity { ClaimsList = new List<StringClaim>() };
                    foreach (Claim c in aggregateIdentity.Claims)
                    {
                        identity.ClaimsList.Add(new StringClaim { ClaimType = c.Type, Value = c.Value });
                    }
                    claimOutput.AggregateClaims.Add(new Identity { ClaimsList = identity.ClaimsList });
                }
            }

            return claimOutput;
        }

        public MofTD GetMofTD()
        {
            MofTD motd = new MofTD();

            ClaimsPrincipal principal = Thread.CurrentPrincipal as ClaimsPrincipal;

            if (principal != null)
            {
                StringBuilder outSb = new StringBuilder("");

                // Also see GetDevice/Title/AggregateIdentity helpers
                ClaimsIdentity primaryUser = principal.GetPrimaryUserIdentity();
                IEnumerable<ClaimsIdentity> userIdentities = principal.GetUserIdentities();

                bool primaryExists = primaryUser != null;
                bool hasUsers = userIdentities.Any();

                if (primaryExists)
                {
                    string primaryGamertag;
                    if (primaryUser.TryGetClaimValue(AuthClaimTypes.Gamertag, out primaryGamertag))
                    {
                        outSb.AppendFormat("Hello {0}! You're the boss today! ", primaryGamertag);
                    }
                }

                if (hasUsers)
                {
                    foreach (ClaimsIdentity userIdentity in userIdentities)
                    {
                        string gamertag;

                        if ( (!primaryExists || (userIdentity.Name != primaryUser.Name)) &&
                            userIdentity.TryGetClaimValue(AuthClaimTypes.Gamertag, out gamertag))
                        {
                            outSb.AppendFormat("Hello {0}! ", gamertag);
                        }
                    }
                }
                else
                {
                    outSb.AppendFormat("No users attached to this token :( ");
                }

                outSb.Append("Today is ");
                outSb.Append(DateTime.Today.DayOfWeek);
                outSb.Append(". Have a nice day!");

                // return JSON Message of the Day response
                motd.MessageOfTheDay = outSb.ToString();
            }

            else
            {
                motd.MessageOfTheDay = "No good token was received :(";
            }

            return motd;
        }
    }
}